﻿using System;
using System.IO;
using System.Text;
using System.Web.Http;
using Dashboard.Data;
using Dashboard.HostMessaging;
using Dashboard.Results;
using Microsoft.Azure.Jobs;
using Microsoft.Azure.Jobs.Protocols;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;

namespace Dashboard.ApiControllers
{
    [Route("api/log/{action}/{id?}", Name="LogControllerRoute")]
    public class LogController : ApiController
    {
        private readonly IFunctionInstanceLookup _functionInstanceLookup;
        private readonly CloudStorageAccount _account;

        internal LogController(IFunctionInstanceLookup functionInstanceLookup, CloudStorageAccount account)
        {
            _functionInstanceLookup = functionInstanceLookup;
            _account = account;
        }

        [HttpGet]
        public IHttpActionResult Output(string id, int start = 0)
        {
            // Parse the ID
            Guid funcId;
            if (!Guid.TryParse(id, out funcId))
            {
                return BadRequest();
            }

            // Get the invocation log
            var instance = _functionInstanceLookup.Lookup(funcId);
            if (instance == null)
            {
                return NotFound();
            }

            LocalBlobDescriptor outputBlobDescriptor = instance.OutputBlob;

            if (outputBlobDescriptor == null)
            {
                return NotFound();
            }

            CloudBlockBlob blob = outputBlobDescriptor.GetBlockBlob(_account);
            
            var sb = new StringBuilder();
            using (var stream = blob.OpenRead())
            {
                using (var sr = new StreamReader(stream))
                {
                    string line = sr.ReadLine();
                    for (int i = 0; line != null; i++, line = sr.ReadLine())
                    {
                        if (i >= start)
                        {
                            sb.AppendLine(line);
                        }
                    }
                }
            }

            return new TextResult(sb.ToString(), Request);
        }

        [HttpGet]
        public IHttpActionResult Blob(string path)
        {
            if (String.IsNullOrEmpty(path))
            {
                return Unauthorized();
            }

            var p = new CloudBlobPath(path);

            if (_account == null)
            {
                return Unauthorized();
            }

            var blob = p.Resolve(_account);

            // Get a SAS for the next 10 mins
            string sas = blob.GetSharedAccessSignature(new SharedAccessBlobPolicy
            {
                Permissions = SharedAccessBlobPermissions.Read,
                SharedAccessExpiryTime = DateTime.UtcNow.AddMinutes(10)
            });

            // Redirect to it
            return Redirect(blob.Uri.AbsoluteUri + sas);
        }
    }
}
